@@ -32,7 +32,7 @@ import hashlib
 _ = gettext.gettext
 
 LOG_FILENAME = 'arduino.out'
-logging.basicConfig(filename=LOG_FILENAME,level=logging.DEBUG)
+#logging.basicConfig(filename=LOG_FILENAME,level=logging.DEBUG)
 _debug = 1
 
 import board
@@ -122,8 +122,8 @@ def compile(tw, id, output, notify):
 	if cont is "": return -1
 	context = notify.get_context_id("main")
 	notify.pop(context)
-	notify.push(context, _("Compilling..."))
-	if _debug: sys.stderr.write('start compile\n')
+	notify.push(context, _("Compiling..."))
+	misc.printLogMessageLn('Compile start')
 	misc.clearConsole(output)
 	tmpdir = id
 	tempobj = tempfile.mktemp("", "Tempobj", id)
@@ -135,7 +135,7 @@ def compile(tw, id, output, notify):
 		"""preproces pde"""
 		(pre_file, lines) = preproc.addHeaders(id, buf)
 		"""compile C targets"""
-		if p.getValue("build.verbose"): sys.stderr.write('process C targets\n')
+		misc.printLogMessageLn('processing C targets')
 		for i in cobj:
 			compline=[j for j in defc]
 			compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
@@ -143,14 +143,14 @@ def compile(tw, id, output, notify):
 			compline.append("-I"+misc.getArduinoPath())
 			compline.append(os.path.join(misc.getArduinoPath(), i))
 			compline.append("-o"+id+"/"+i+".o")
-			if p.getValue("build.verbose"): sys.stderr.write(' '.join(compline)+"\n")
+			misc.printLogMessageLn(' '.join(compline))
 			(run, sout) = misc.runProg(compline)
-			if p.getValue("build.verbose"): sys.stderr.write(sout+"\n")
+			misc.printLogMessageLn(sout)
 			if run == False:
 				misc.printError(notify, output, stripOut(sout, pre_file))
-				raise
+				raise NameError("compile error")
 		"""compile C++ targets"""
-		if _debug: sys.stderr.write('process C++ targets\n')
+		misc.printLogMessageLn('processing C++ targets')
 		for i in cppobj:
 			compline = [j for j in defcpp]
 			compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
@@ -158,111 +158,127 @@ def compile(tw, id, output, notify):
 			compline.append("-I" + misc.getArduinoPath())
 			compline.append(os.path.join(misc.getArduinoPath(), i))
 			compline.append("-o"+id+"/"+i+".o")
-			if p.getValue("build.verbose"): sys.stderr.write(' '.join(compline)+"\n")
+			misc.printLogMessageLn(' '.join(compline))
 			(run, sout) = misc.runProg(compline)
-			if p.getValue("build.verbose"): sys.stderr.write(sout+"\n")
+			misc.printLogMessageLn(sout)
 			if run == False:
 				misc.printError(notify, output, sout)
-				raise
+				raise NameError("compile error")
 		"""generate archive objects"""
-		if _debug: sys.stderr.write('gen ar obj\n')
+		misc.printLogMessageLn('generating ar objects')
 		for i in cobj+cppobj:
 			compline = [j for j in defar]
 			compline.append(id+"/core.a")
 			compline.append(id+"/"+i+".o")
-			if p.getValue("build.verbose"): sys.stderr.write(' '.join(compline)+"\n")
+			misc.printLogMessageLn(' '.join(compline))
 			(run, sout) = misc.runProg(compline)
-			if p.getValue("build.verbose"): sys.stderr.write(sout+"\n")
+			misc.printLogMessageLn(sout)
 			if run == False:
 				misc.printError(notify, output, stripOut(sout, pre_file))
-				raise
+				raise NameError("compile error")
 		"""precompile pde"""
-		if _debug: sys.stderr.write('pde compile\n')
-		if _debug: sys.stderr.write('-----------\n')
+		misc.printLogMessageLn('pde compile')
+		misc.printLogMessageLn('-----------')
 		compline=[j for j in defcpp]
 		compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
 		compline.append("-DF_CPU="+b.getBoardFCPU(b.getBoard()))
 		compline.append("-I" + misc.getArduinoPath())
-		compline.extend(preproc.generateCFlags(id, cont))
+		flags = []
+		flags = preproc.generateCFlags(id, cont)
+		compline.extend(flags)
 		compline.extend(["-I" + os.path.join(i, "utility") for i in preproc.generateLibs(id, buf)])
 		compline.append(pre_file)
 		compline.append("-o"+pre_file+".o")
-		if p.getValue("build.verbose"): sys.stderr.write(' '.join(compline)+"\n")
+		misc.printLogMessageLn(' '.join(compline))
 		(run, sout) = misc.runProg(compline)
-		if p.getValue("build.verbose"): sys.stderr.write(sout+"\n")
+		misc.printLogMessageLn(sout)
 		if run == False:
 			misc.printError(notify, output, stripOut(sout, pre_file))
 			moveCursor(tw, int(getErrorLine(sout, lines)))
 			raise NameError('compile-error')
 
 		"""compile all objects"""
-		if _debug: sys.stderr.write('compile objects\n')
-		if _debug: sys.stderr.write('---------------\n')
+		misc.printLogMessageLn('compile objects')
+		misc.printLogMessageLn('---------------')
 		compline = [i for i in link]
 		compline.append("-mmcu="+b.getBoardMCU(b.getBoard()))
 		compline.append("-o"+tempobj+".elf")
 		compline.append(pre_file+".o")
 		tmplibs = []
 		for i in preproc.generateLibs(id, buf):
-			tmplibs.extend(validateLib(os.path.basename(i), tempobj, output, notify))
+			tmplibs.extend(validateLib(os.path.basename(i), tempobj, flags, output, notify))
 		compline.extend(list(set(tmplibs)))
 		compline.extend(["-I" + os.path.join(i, "utility") for i in preproc.generateLibs(id, buf)])
 		compline.append(id+"/core.a")
 		compline.append("-L"+id)
 		compline.append("-lm")
-		if p.getValue("build.verbose"): sys.stderr.write(' '.join(compline)+"\n")
+		misc.printLogMessageLn(' '.join(compline))
 		(run, sout) = misc.runProg(compline)
-		if p.getValue("build.verbose"): sys.stderr.write(sout+"\n")
+		misc.printLogMessageLn(sout)
 		if run == False:
 			misc.printError(notify, output, stripOut(sout, pre_file))
 			raise NameError('linking-error')
 		compline=[i for i in eep]
 		compline.append(tempobj+".elf")
 		compline.append(tempobj+".eep")
-		if p.getValue("build.verbose"): sys.stderr.write(' '.join(compline)+"\n")
+		misc.printLogMessageLn(' '.join(compline))
 		(run, sout) = misc.runProg(compline)
-		if p.getValue("build.verbose"): sys.stderr.write(sout+"\n")
+		misc.printLogMessageLn(sout)
 		if run == False:
 			misc.printError(notify, output, stripOut(sout, pre_file))
 			raise NameError('obj-copy')
 		compline=[i for i in hex]
 		compline.append(tempobj+".elf")
 		compline.append(tempobj+".hex")
-		if p.getValue("build.verbose"): sys.stderr.write(' '.join(compline)+"\n")
+		misc.printLogMessageLn(' '.join(compline))
 		(run, sout) = misc.runProg(compline)
-		if p.getValue("build.verbose"): sys.stderr.write(sout+"\n")
+		misc.printLogMessageLn(sout)
 		if run == False:
 			misc.printError(notify, output, stripOut(sout, pre_file))
 			raise NameError('obj-copy')
 		size = computeSize(tempobj+".hex")
 		notify.pop(context)
 		notify.push(context, _("Done compilling."))
-		if _debug: sys.stderr.write("compile done.\n")
+		misc.printLogMessageLn("compile done.")
 
 		misc.printMessage(output, \
 			_("Binary sketch size: %s bytes (of a %s bytes maximum)") % (size, b.getBoardMemory(b.getBoard())))
 	except StandardError as e:
 		print "Error: %s" % e
-	except:
+		return -1
+	except Exception as e:
 		print "Error compiling. Op aborted!"
+		print "Error: %s" % e
+		return -1
 	return tempobj
 
+def getLibraries():
+	paths = ["", misc.getArduinoPath()]
+	dirs = ["", "utility"]
+	for d in dirs:
+		for q in paths:
+			print q
+	#		fl = os.path.join(q, library, d)
+	#		if os.path.exists(fl):
+
 """checks whether library exists (it has been compiled and tries to compile it otherwise"""
 """@returns a list of compiled objects"""
-def validateLib(library, tempobj, output, notify):
+def validateLib(library, tempobj, flags, output, notify):
 	"""compile library also try to compile every cpp under libdir"""
 	"""also try to compile utility dir if present"""
-	paths = ["", misc.getArduinoPath()]
+	paths = ["", misc.getArduinoPath(), misc.getArduinoLibsPath()]
 	if config.user_library != None and config.user_library != -1:
 		paths.extend(i.strip() for i in config.user_library.split(';'))
 	dirs = ["", "utility"]
 	b = board.Board()
 	res = []
 	fl = []
+	queue = []
 	for d in dirs:
 		for q in paths:
 			fl = os.path.join(q, library, d)
-			if os.path.exists(fl):
+			if fl not in set(queue) and os.path.exists(fl):
+				queue.append(fl)
 				try:
 					for i in glob.glob(os.path.join(fl, "*.c")):
 						"""compile library c modules"""
@@ -272,12 +288,13 @@ def validateLib(library, tempobj, output, notify):
 						compline.append("-I" + misc.getArduinoPath())
 						compline.append("-I" + os.path.join(misc.getArduinoLibsPath(), library, "utility"))
 						compline.extend(preproc.generateCFlags(id, open(i).read()))
+						compline.extend(flags)
 						compline.append(i)
 						compline.append("-o"+os.path.join(os.path.dirname(tempobj), \
 							os.path.basename(i.replace(".c", ".o"))))
-						if p.getValue("build.verbose"): sys.stderr.write(' '.join(compline)+"\n")
+						misc.printLogMessageLn(' '.join(compline))
 						(run, sout) = misc.runProg(compline)
-						if p.getValue("build.verbose"): sys.stderr.write(sout+"\n")
+						misc.printLogMessageLn(sout)
 						if run == False:
 							misc.printError(notify, output, sout)
 							raise NameError('libs compile error')
@@ -293,13 +310,14 @@ def validateLib(library, tempobj, output, notify):
 						compline.append("-DF_CPU="+b.getBoardFCPU(b.getBoard()))
 						compline.append("-I" + misc.getArduinoPath())
 						compline.extend(preproc.generateCFlags(id, open(i).read()))
+						compline.extend(flags)
 						compline.append(i)
 						compline.append("-I" + os.path.join(misc.getArduinoLibsPath(), library, "utility"))
 						compline.append("-o"+os.path.join(os.path.dirname(tempobj), \
 							os.path.basename(i.replace(".cpp", ".o"))))
-						if p.getValue("build.verbose"): sys.stderr.write(' '.join(compline)+"\n")
+						misc.printLogMessageLn(' '.join(compline))
 						(run, sout) = misc.runProg(compline)
-						if p.getValue("build.verbose"): sys.stderr.write(sout+"\n")
+						misc.printLogMessageLn(sout)
 						if run == False:
 							misc.printError(notify, output, sout)
 							raise NameError('libs compile error')
