@@ -29,15 +29,40 @@ import gettext
 import sys
 _ = gettext.gettext
 
+import config
 import prefs
 
 LOG_FILENAME = 'arduino.out'
-logging.basicConfig(filename=LOG_FILENAME,level=logging.DEBUG)
+#logging.basicConfig(filename=LOG_FILENAME,level=logging.DEBUG)
 
+defaultPath = os.path.expanduser('~/.arduino')
 arduino_path = "hardware/arduino/cores/arduino"
 arduino_boot_path = "hardware/arduino/bootloaders"
 arduino_libs_path = "libraries"
 
+def getArduinoFile(filename):
+	path = None
+	try:
+		path = os.path.join(os.getcwd(), defaultPath, filename)
+		if os.path.exists(path):
+			return path
+		else: raise Exception
+	except:
+		try:
+			path = os.path.join(sys.prefix, "local", "share", "gnoduino", filename)
+			if os.path.exists(path):
+				return path
+			else: raise Exception
+		except:
+			try:
+				path = os.path.join(sys.prefix, "share", "gnoduino", filename)
+				if os.path.exists(path):
+					return path
+				else: raise Exception
+			except Exception,e:
+				print(e)
+				return None
+
 def getArduinoPath():
 	try:
 		path = os.path.join(os.getcwd(), arduino_path)
@@ -79,6 +104,36 @@ def getArduinoLibsPath():
 		except: path = ""
 	finally: return path
 
+def getArduinoUiPath():
+	try:
+		path = os.path.join(os.getcwd(), "ui")
+		if os.path.exists(path) is False: raise
+	except:
+		try:
+			path = os.path.join(sys.prefix, "local", "share", "gnoduino", "ui")
+			if os.path.exists(path) is False: raise
+		except:
+			try:
+				path = os.path.join(sys.prefix, "share", "gnoduino", "ui")
+				if os.path.exists(path) is False: raise
+			except: path = ""
+	finally: return path
+
+def getPixmapPath(pixmap):
+	try:
+		path = os.path.join(os.getcwd(), "pixmaps", pixmap)
+		if os.path.exists(path):
+			return path
+		else: raise NameError("System error")
+	except:
+		try:
+			path = os.path.join(sys.prefix, 'share', 'gnoduino', "pixmaps", pixmap)
+			if os.path.exists(path):
+				return path
+			else: raise NameError("System error")
+		except Exception,e:
+			print(e)
+			raise SystemExit(_("Cannot load pixmap files"))
 
 def makeWorkdir():
 	return tempfile.mkdtemp("", os.path.join(tempfile.gettempdir(), "build"+str(time.time())))
@@ -103,6 +158,9 @@ def runProg(cmdline):
 				gtk.main_iteration()
 			sout += o
 		if p.poll()==1: raise
+	except OSError as e:
+		err = "ERROR: Error running %s: %s" % (cmdline[0], e.strerror)
+		return (False, err)
 	except:
 		logging.debug("ERR:%s", sout)
 		return (False, sout)
@@ -126,7 +184,7 @@ def runProgOutput(console, cmdline):
 			if o == '' and p.poll() != None: break
 			printMessageLn(console, o)
 			pr = prefs.preferences()
-			if pr.getValue("build.verbose"): sys.stderr.write(o)
+			if pr.getBoolValue("build.verbose"): sys.stderr.write(o)
 			sout += o
 		if p.poll()==1: raise
 	except:
@@ -134,6 +192,14 @@ def runProgOutput(console, cmdline):
 		return False
 	return True
 
+def merge_font_name(widget, font):
+	if widget == None: return
+	context = widget.get_pango_context()
+	font_desc = context.get_font_description()
+	cur_font = pango.FontDescription(font)
+	font_desc.merge(cur_font, True)
+	return font_desc.to_string()
+
 def set_widget_font(widget, font):
 	if widget == None: return
 	context = widget.get_pango_context()
@@ -170,6 +236,9 @@ def printMessageLn(console, message):
 	mark = b.get_insert()
 	console.scroll_mark_onscreen(mark)
 
+def printLogMessageLn(message):
+	if config.build_verbose == 'true': sys.stderr.write(message+"\n")
+
 def statusMessage(w, text):
 	w.push(1, text)
 	glib.timeout_add(1000, lambda x: w.pop(1), 0)
@@ -196,8 +265,12 @@ def createPopup(title, parent, msg):
 	dialog = gtk.MessageDialog(parent,
 			gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
 			gtk.MESSAGE_WARNING,
-			gtk.BUTTONS_YES_NO,
+			gtk.BUTTONS_NONE,
 			msg)
+	dialog.add_buttons(_("Close without Saving"), 1, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+			gtk.STOCK_SAVE_AS, gtk.RESPONSE_YES)
+	dialog.format_secondary_text(_("If you don't save, changes will be permanently lost."))
+	dialog.set_default_response(gtk.RESPONSE_CANCEL)
 	response = dialog.run()
 	dialog.destroy()
 	return response
