@@ -101,9 +101,9 @@ def findText(widget, event, data=None):
 		b = view.get_buffer()
 		mark = b.get_insert()
 		iter = b.get_iter_at_mark(mark)
-		search = widget.get_active_text()
+		search = widget.get_text()
 		flags = 0
-		if data[0].get_active() == True:
+		if data[0].get_active() == False:
 			flags = gtksourceview2.SEARCH_CASE_INSENSITIVE
 		backwards = False
 		if data[2].get_active() == True:
@@ -178,14 +178,18 @@ def createsrcview(status, f=None):
 		sbuffer.set_language(manager.get_language("c"))
 	sv = gtksourceview2.View(sbuffer)
 	p = prefs.preferences()
-	misc.set_widget_font(sv, p.getValue("editor.font").replace(",", " "))
+	misc.set_widget_font(sv, config.cur_editor_font)
 	manager = gtksourceview2.StyleSchemeManager()
 #	for i in gtksourceview2.StyleSchemeManager.get_scheme_ids(manager):
 #		print i
-	scheme =  manager.get_scheme("tango")
+	manager.append_search_path(misc.getArduinoUiPath())
+	manager.force_rescan()
+	scheme =  manager.get_scheme("arduino")
 	sbuffer.set_style_scheme(scheme);
-	sv.set_size_request(500, 450)
+	sv.set_size_request(300, 100)
 	sv.set_editable(True)
+	if config.show_numbers == 'true':
+		sv.set_show_line_numbers(True)
 	sv.set_cursor_visible(True)
 #	sv.set_show_line_numbers(True)
 	sv.set_wrap_mode(gtk.WRAP_CHAR)
