@@ -17,13 +17,13 @@
 
 import os
 import sys
-import gconf
 import glib
 import gobject
 import gtk
 import subprocess
 import select
 import shutil
+import signal
 
 import gettext
 _ = gettext.gettext
@@ -42,14 +42,8 @@ import gnoduino
 import gtksourceview2
 font = "Monospace 10"
 
-MW = "/apps/gnoduino/width"
-MH = "/apps/gnoduino/height"
-user_library_key = "/apps/gnoduino/user_library"
-serial_port_key = "/apps/gnoduino/serial_port"
-serial_baud_rate_key = "/apps/gnoduino/serial_baud_rate"
-
 def setupPage(w, page, p):
-	misc.set_widget_font(getCurrentView(), config.cur_font)
+	misc.set_widget_font(getCurrentView(), config.cur_editor_font)
 	getCurrentPage().queue_resize()
 	pg = w.get_nth_page(p)
 	cl = pg.get_data("close");
@@ -66,13 +60,18 @@ def destroyPage(w, b):
 	f = b.get_data("file")
 	buf = b.get_data("buffer")
 	if misc.bufferModified(buf, f) is True:
-		save = misc.createPopup("Save document", mainwin, \
-			_("Save changes to document\n%s\n before closing?" % f))
+		if f is None: f= "Untitled"
+		save = misc.createPopup(_("Save document"), mainwin, \
+			_("Save changes to document \"%s\"\n before closing?" % os.path.basename(f)))
 		if save == gtk.RESPONSE_YES:
-			csave(None, False)
+			if csave(None, False) is False:
+				return False
+		if save == gtk.RESPONSE_CANCEL or save == gtk.RESPONSE_DELETE_EVENT:
+			return False
 	nb.remove_page(nb.page_num(b))
 	if nb.get_n_pages() < 1:
 		createPage(nb)
+	return True
 
 def updatePageTitle(w, status):
 	page = getCurrentPage()
@@ -85,7 +84,7 @@ def updatePageTitle(w, status):
 		name = "Untitled"
 
 	if (misc.bufferModified(w, f)):
-		l.set_text(name+"*")
+		l.set_text("*"+name)
 	else:
 		l.set_text(name)
 	srcview.updatePos(w, status)
@@ -148,20 +147,37 @@ def searchFile(nb, f):
 			return True
 	return False
 
-def processFile(w):
-	if searchFile(nb, w.get_filename()) == True:
+def openFile(w):
+	f = w.get_filename()
+	if searchFile(nb, f) == True:
 		w.destroy()
 		return
+	processFile(f)
+
+def processFile(f):
+	if not os.path.exists(f):
+		return
 	page = getCurrentPage()
-	createPage(nb, w.get_filename())
+	createPage(nb, f)
 	if page.get_data("label").get_text() == "Untitled":
 		replacePage(page)
 
 def saveAs():
-	p = gtk.FileChooserDialog("Save file", None, gtk.FILE_CHOOSER_ACTION_OPEN,
+	page = getCurrentPage()
+	buf = page.get_data("buffer")
+	f = page.get_data("file")
+	if misc.bufferModified(buf, f) is False:
+		return
+	p = gtk.FileChooserDialog("Save file", None, gtk.FILE_CHOOSER_ACTION_SAVE,
 		(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
 		gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-	p.set_size_request(650, 500)
+	if page.get_data("file") is not None:
+		name =  os.path.basename(page.get_data("file"))
+		p.set_filename(name)
+	else:
+		name = "Untitled"
+		p.set_current_name(name)
+	p.set_default_size(450, 400)
 	p.show_all()
 	if p.run() == gtk.RESPONSE_ACCEPT:
 		f = p.get_filename()
@@ -174,10 +190,10 @@ def copen(widget, data=None):
 	p = gtk.FileChooserDialog("Open file", None, gtk.FILE_CHOOSER_ACTION_OPEN,
 		(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
 		gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
-	p.set_size_request(650, 500)
+	p.set_size_request(450, 400)
 	p.show_all()
 	if p.run() == gtk.RESPONSE_ACCEPT:
-		processFile(p)
+		openFile(p)
 	p.destroy()
 
 def csave_as(w, data=None):
@@ -190,7 +206,7 @@ def csave(w, data=None):
 	f = page.get_data("file")
 	if f == None or data == True:
 		f = saveAs()
-		if f == None: return
+		if f == None: return False
 		else:
 			if os.path.exists(f):
 				p = misc.MessageBox()
@@ -203,15 +219,19 @@ def csave(w, data=None):
 	F.write(b.get_text(b.get_start_iter(), b.get_end_iter()))
 	F.close()
 	updatePageTitle(b, sb2)
+	return True
 
 def quit(widget, data=None):
+	for i in range(nb.get_n_pages()):
+		if (destroyPage(None, nb.get_nth_page(0)) is False):
+			return True
 	shutil.rmtree(id, True)
 	gtk.main_quit()
 
 def find(widget, data=None):
 	find = gui.get_object("find")
 	find_text = gui.get_object("find-text")
-	find_text.child.select_region(0, -1)
+	find_text.select_region(0, -1)
 	cbs = ["checkbutton1", "checkbutton2","checkbutton3", "checkbutton4"]
 	find_text.connect("key-release-event", srcview.findText, [gui.get_object(i) for i in cbs])
 	find.set_default_response(gtk.RESPONSE_OK)
@@ -220,6 +240,9 @@ def find(widget, data=None):
 		srcview.findText(find_text, -1, [gui.get_object(i) for i in cbs])
 	find.hide()
 
+def libImport(widget, data=None):
+	compiler.getLibraries()
+
 def compile(widget, data=file):
 	cserial(None, 0, sctw)
 	page = getCurrentPage()
@@ -233,6 +256,13 @@ def upload(widget, serial, data=file):
 		gtk.main_iteration()
 	uploader.upload(obj, serial, tw, sb)
 
+def butSave(widget, data=None):
+	b = getCurrentPage()
+	f = b.get_data("file")
+	buf = b.get_data("buffer")
+	if misc.bufferModified(buf, f) is True:
+		csave(widget, False)
+
 def menuUpload(widget, data=None):
 	upload(widget, ser, data)
 
@@ -246,34 +276,63 @@ def menuResetBoard(widget, data=None):
 	ser.resetBoard()
 
 def preferences(widget, data=None):
-	global client
 	pref = gui.get_object("preferences")
-	fs = gui.get_object("fontsize")
-	p = prefs.preferences()
-	if config.cur_font != -1:
-		fs.set_value(float(config.cur_font.split(",")[2]))
-	else:
-		fs.set_value(float(p.getValue("editor.font").split(",")[2]))
+	fe = gui.get_object("fontbutton1")
+	fc = gui.get_object("fontbutton2")
 	bv = gui.get_object("build.verbose")
-	if config.build_verbose != -1:
-		bv.set_active(bool(config.build_verbose))
-	else:
-		bv.set_active(bool(p.getValue("build.verbose")))
+	sn = gui.get_object("show.numbers")
 	ul = gui.get_object("user.library")
+	p = prefs.preferences()
+	fe.set_font_name(misc.merge_font_name(fe, config.cur_editor_font))
+	fc.set_font_name(misc.merge_font_name(fc, config.cur_console_font))
+	val = 0
+	if config.build_verbose != -1:
+		if config.build_verbose == 'true':
+			val = 1
+	else:
+		if p.getSafeValue("build.verbose", "false") == 'true':
+			val = 1
+	bv.set_active(val)
+	val = 0
+	if config.show_numbers != -1:
+		if config.show_numbers == 'true':
+			val = 1
+	else:
+		if p.getSafeValue("show.numbers", "false") == 'true':
+			val = 1
+	sn.set_active(val)
 	if (config.user_library != None and config.user_library != -1):
 		ul.set_text(str(config.user_library))
 	else:
-		ul.set_text(str(client.get_string(user_library_key)) if client.get_string(user_library_key) != None else "")
+		ul.set_text(str(p.getSafeValue("user.library", "")))
 	r = pref.run()
 	if r == 1:
-		config.cur_font =  p.getValue("editor.font").split(",")[0] + \
-			"," + p.getValue("editor.font").split(",")[1] + \
-			"," + str(int(fs.get_value()))
-		misc.set_widget_font(tw, config.cur_font)
-		misc.set_widget_font(sctw, config.cur_font)
-		misc.set_widget_font(getCurrentView(), config.cur_font)
+		config.cur_editor_font = fe.get_font_name()
+		p.setValue("editor.font", config.cur_editor_font.replace(" ", ","))
+		config.cur_console_font = fc.get_font_name()
+		p.setValue("console.font", config.cur_console_font.replace(" ", ","))
 		config.user_library = ul.get_text()
-		client.set_string(user_library_key, config.user_library)
+		if bv.get_active() == 1:
+			config.build_verbose = 'true'
+		else:
+			config.build_verbose = 'false'
+		p.setValue("build.verbose", config.build_verbose)
+		if sn.get_active() == 1:
+			config.show_numbers = 'true'
+		else:
+			config.show_numbers = 'false'
+		p.setValue("show.numbers", config.show_numbers)
+		for i in range(nb.get_n_pages()):
+			sv = nb.get_nth_page(i).get_data("view")
+			if (config.show_numbers == 'true'):
+				sv.set_show_line_numbers(True)
+			else:
+				sv.set_show_line_numbers(False)
+		p.setValue("user.library", config.user_library)
+		p.saveValues()
+		misc.set_widget_font(tw, config.cur_console_font)
+		misc.set_widget_font(sctw, config.cur_console_font)
+		misc.set_widget_font(getCurrentView(), config.cur_editor_font)
 	pref.hide()
 
 def stop(widget, data=None):
@@ -290,6 +349,9 @@ def cserial(w, st, data=None):
 			vbox.add(con)
 		return
 	if (sertime == None):
+		if not ser.serial.isOpen():
+			ser.getConfigSerialPort(sb, tw)
+			return
 		sertime = glib.timeout_add(1000,
 			ser.updateConsole,
 			data)
@@ -305,17 +367,19 @@ def burnBootloader(w, id):
 	uploader.burnBootloader(ser, tw, sb, id)
 
 def setBaud(w, data=None):
-	ser.resetBoard()
-	ser.serial.close()
+	if ser.serial.isOpen():
+		ser.resetBoard()
+		ser.serial.close()
+	else: return
 	if config.serial_baud_rate == -1:
-		config.serial_baud_rate = client.get_string(serial_baud_rate_key)
-		if config.serial_baud_rate: defbaud = config.serial_baud_rate
-		else: defbaud = p.getValue("serial.debug_rate")
+		config.serial_baud_rate = p.getSafeValue("serial.debug_rate", p.getDefaultValue("serial.debug_rate"))
+		defbaud = config.serial_baud_rate
 	else:
 		if w: defbaud = ser.getBaudrates()[w.get_active()]
 	ser.serial.baudrate = [i for i in ser.getBaudrates() if i == defbaud][0]
 	ser.serial.open()
-	client.set_string(serial_baud_rate_key, defbaud)
+	p.setValue("serial.debug_rate", defbaud)
+	p.saveValues()
 
 def serSendText(w, data=None):
 	ser.serial.write(w.get_text())
@@ -331,6 +395,27 @@ def redo(w, data=None):
 	b = page.get_data("buffer")
 	if b.can_redo(): b.redo()
 
+def cut(w, data=None):
+	page = getCurrentPage()
+	b = page.get_data("buffer")
+	display = gtk.gdk.display_manager_get().get_default_display()
+	clipboard = gtk.Clipboard(display, "PRIMARY")
+	b.cut_clipboard(clipboard, True)
+
+def copy(w, data=None):
+	page = getCurrentPage()
+	b = page.get_data("buffer")
+	display = gtk.gdk.display_manager_get().get_default_display()
+	clipboard = gtk.Clipboard(display, "PRIMARY")
+	b.copy_clipboard(clipboard)
+
+def paste(w, data=None):
+	page = getCurrentPage()
+	b = page.get_data("buffer")
+	display = gtk.gdk.display_manager_get().get_default_display()
+	clipboard = gtk.Clipboard(display, "PRIMARY")
+	b.paste_clipboard(clipboard, None, True)
+
 menus = [
 		("menu-new", cnew, (ord('n'), gtk.gdk.CONTROL_MASK)),
 		("menu-open", copen, (ord('o'), gtk.gdk.CONTROL_MASK)),
@@ -340,7 +425,11 @@ menus = [
 		("menu-find", find, (ord('f'), gtk.gdk.CONTROL_MASK)),
 		("menu-undo", undo, (ord('z'), gtk.gdk.CONTROL_MASK)),
 		("menu-redo", redo, (ord('z'), gtk.gdk.CONTROL_MASK|gtk.gdk.SHIFT_MASK)),
+		("menu-cut", cut, (ord('x'), gtk.gdk.CONTROL_MASK)),
+		("menu-copy", copy, (ord('c'), gtk.gdk.CONTROL_MASK)),
+		("menu-paste", paste, (ord('v'), gtk.gdk.CONTROL_MASK)),
 		("menu-compile", compile, (ord('r'), gtk.gdk.CONTROL_MASK)),
+		("menu-import", libImport, (ord('i'), gtk.gdk.CONTROL_MASK)),
 		("menu-reset-board", menuResetBoard, (ord('m'), gtk.gdk.CONTROL_MASK)),
 		("menu-preferences", preferences, (None, None)),
 		("menu-upload", menuUpload, (ord('u'), gtk.gdk.CONTROL_MASK)),
@@ -354,19 +443,20 @@ def menu(gui):
 	[gui.get_object(i[0]).add_accelerator("activate", \
 		accel, i[2][0], i[2][1], 0) for i in menus if i[2][0] != None]
 	mainwin.add_accel_group(accel)
+	gui.get_object("menu-find-next").set_sensitive(False)
+	gui.get_object("menu-replace").set_sensitive(False)
 
 def createCon():
 	sw = gtk.ScrolledWindow()
 	sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 	sw.set_shadow_type(gtk.SHADOW_IN)
 	tmp = gtk.TextView()
-	tmp.set_size_request(-1, 150)
 	tmp.set_editable(False)
 	twbuf = gtk.TextBuffer()
 	tmp.set_buffer(twbuf)
 	tmp.modify_base(gtk.STATE_NORMAL, gtk.gdk.Color(0,0,0))
 	tmp.modify_text(gtk.STATE_NORMAL, gtk.gdk.Color("#ffffff"))
-	misc.set_widget_font(tmp, p.getValue("editor.font").replace(",", " "))
+	misc.set_widget_font(tmp, config.cur_console_font)
 	sw.add(tmp)
 	sw.show_all()
 	return (sw, tmp)
@@ -377,13 +467,12 @@ def createScon():
 	sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 	sw.set_shadow_type(gtk.SHADOW_IN)
 	tmp = gtk.TextView()
-	tmp.set_size_request(-1, 150)
 	tmp.set_editable(False)
 	twbuf = gtk.TextBuffer()
 	tmp.set_buffer(twbuf)
 	tmp.modify_base(gtk.STATE_NORMAL, gtk.gdk.Color(0,0,0))
 	tmp.modify_text(gtk.STATE_NORMAL, gtk.gdk.Color("#ffffff"))
-	misc.set_widget_font(tmp, p.getValue("editor.font").replace(",", " "))
+	misc.set_widget_font(tmp, config.cur_console_font)
 	sw.add(tmp)
 	hbox = gtk.HBox(False, 0)
 	s = gtk.Button("Send")
@@ -411,7 +500,7 @@ buttons = [
 		("open", "open.png", copen),
 		("new", "new.png", cnew),
 
-		("save", "save.png", None),
+		("save", "save.png", butSave),
 		("upload", "upload.png", None),
 		("serial", "serial.png", None)
 	]
@@ -436,12 +525,17 @@ def setSerial(w, id):
 	createBaudCombo(w, id)
 	try:
 		config.cur_serial_port = id
-		ser.resetBoard()
-		ser.serial.close()
+		if ser.serial.isOpen():
+			ser.resetBoard()
+			ser.serial.close()
 		ser.serial.port = id
 		ser.serial.open()
-		client.set_string(serial_port_key, id)
-	except: None
+		p.setValue("serial.port", id)
+		p.saveValues()
+	except Exception,e:
+		misc.clearConsole(tw)
+		misc.printMessage(tw, str(e))
+		print(e)
 
 def getCurrentPage():
 	return nb.get_nth_page(nb.get_current_page())
@@ -455,8 +549,14 @@ def getGui():
 	return gui
 
 def cb_configure_event(widget, event):
-	client.set_int(MW, event.width)
-	client.set_int(MH, event.height)
+	p.setValue("default.window.width", event.width)
+	p.setValue("default.window.height", event.height)
+	p.saveValues()
+
+def vbox_move_handle(widget, scrolltype):
+	p.setValue("console.height", widget.get_position())
+	p.saveValues()
+	return True
 
 def run():
 	try:
@@ -475,8 +575,8 @@ def run():
 		global scon
 		global p
 		global b
-		global client
-		client = gconf.client_get_default()
+		#perform cleanup prior to this
+		signal.signal(signal.SIGINT, signal.SIG_DFL)
 		id = misc.makeWorkdir()
 		ser = serialio.sconsole()
 		p = prefs.preferences()
@@ -486,24 +586,37 @@ def run():
 			path = os.path.join(os.getcwd(), "ui", "main.ui")
 			if os.path.exists(path):
 				gui.add_from_file(path)
-			else: raise
+			else: raise NameError("System error")
 		except:
 			try:
-				path = os.path.join(sys.prefix, "share", "gnoduino", "ui", "main.ui")
+				path = os.path.join(sys.prefix, "local", "share", "gnoduino", "ui", "main.ui")
 				if os.path.exists(path):
 					gui.add_from_file(path)
-			except Exception,e:
-				print(e)
-				raise SystemExit(_("Cannot load ui file"))
+				else: raise NameError("System error")
+			except:
+				try:
+					path = os.path.join(sys.prefix, "share", "gnoduino", "ui", "main.ui")
+					if os.path.exists(path):
+						gui.add_from_file(path)
+					else: raise NameError("System error")
+				except Exception,e:
+					print(e)
+					raise SystemExit(_("Cannot load ui file"))
 		mainwin = gui.get_object("top_win")
-		mainwin.set_default_size(client.get_int(MW), client.get_int(MH))
+		mainwin.set_icon_from_file(misc.getPixmapPath("gnoduino.png"))
+		mw = int(p.getSafeValue("default.window.width", 0))
+		mh = int(p.getSafeValue("default.window.height", 0))
+		if (mw and mh):
+			mainwin.set_default_size(mw, mh)
 		mainwin.connect("configure-event", cb_configure_event)
 		vbox = gui.get_object("vpan")
 		sb = gui.get_object("statusbar1")
 		sb2 = gui.get_object("statusbar2")
-		config.cur_font = p.getValue("editor.font")
-		config.build_verbose = p.getValue("build.verbose")
-		config.user_library = client.get_string(user_library_key)
+		config.cur_editor_font = p.getSafeValue("editor.font", p.getDefaultValue("editor.font")).replace(",", " ")
+		config.cur_console_font = p.getSafeValue("console.font", p.getDefaultValue("console.font")).replace(",", " ")
+		config.build_verbose = p.getSafeValue("build.verbose", p.getDefaultValue("build.verbose"))
+		config.show_numbers = p.getSafeValue("show.numbers", p.getDefaultValue("show.numbers"))
+		config.user_library = p.getValue("user.library")
 		menu(gui)
 		"""build menus"""
 		sub = gtk.Menu()
@@ -516,36 +629,45 @@ def run():
 			menuItem.connect('toggled', selectBoard, i['id'])
 			sub.append(menuItem)
 		gui.get_object("board").set_submenu(sub)
+		(con, tw) = createCon()
 
+		"""setup default serial port"""
 		sub = gtk.Menu()
 		maingroup = gtk.RadioMenuItem(None, None)
-		config.serial_port = client.get_string(serial_port_key)
+		defport = p.getValue("serial.port")
+		validport = False
+		activePort = False
+		"""validate serial ports - this should really be moved to serialio"""
+		for i in ser.scan():
+			if i == defport:
+				validport = True
+
 		for i in ser.scan():
 			menuItem = gtk.RadioMenuItem(maingroup, i)
-			if config.serial_port: defport = config.serial_port
-			else: defport = p.getValue("serial.port")
-			if defport:
+			if defport and validport:
 				if i == defport:
 					menuItem.set_active(True)
 					if config.cur_serial_port == -1:
 						config.cur_serial_port = i
 					setSerial(None, i)
 			else:
-				if i == "/dev/ttyS0":
-					if config.cur_serial_port == -1:
-						config.cur_serial_port = i
-					menuItem.set_active(True)
-					setSerial(None, i)
+				if config.cur_serial_port == -1:
+					config.cur_serial_port = i
+				try:
+					s = ser.tryPort(i)
+					s.close()
+				except: continue
+				menuItem.set_active(True)
+				setSerial(None, i)
 			menuItem.connect('activate', setSerial, i)
 			sub.append(menuItem)
+			activePort = True
 
 		if config.serial_baud_rate == -1:
-			config.serial_baud_rate = client.get_string(serial_baud_rate_key)
-			if config.serial_baud_rate: defbaud = config.serial_baud_rate
-			else: defbaud = p.getValue("serial.debug_rate")
-			config.serial_baud_rate = defbaud
+			config.serial_baud_rate = p.getSafeValue("serial.debug_rate", p.getDefaultValue("serial.debug_rate"))
 
 		gui.get_object("serial_port").set_submenu(sub)
+		gui.get_object("serial_port").set_sensitive(activePort)
 
 		sub = gtk.Menu()
 		pgm = programmer.Programmer()
@@ -560,38 +682,35 @@ def run():
 		sv = createPage(nb)
 		vbox.add(nb)
 
-		(con, tw) = createCon()
 		(scon,sctw) = createScon()
 		vbox.add(con)
-
+		vbox.connect("notify::position", vbox_move_handle)
+		cpos = int(p.getSafeValue("console.height", -1))
+		vbox.set_position(cpos)
 		mainwin.set_focus(sv)
 		mainwin.show_all()
-		mainwin.set_title("Arduino")
-		mainwin.connect("destroy", quit)
+		mainwin.set_title("Gnoduino")
+		mainwin.connect("delete-event", quit)
 		gui.get_object("ser_monitor").connect("activate", cserial, sertime, sctw)
 		gui.get_object("serial").connect("clicked", cserial, sertime, sctw)
 		gui.get_object("upload").connect("clicked", upload, ser)
 		for i in buttons:
 			w = gtk.Image()
-			try:
-				path = os.path.join(os.getcwd(), "pixmaps", i[1])
-				if os.path.exists(path):
-					w.set_from_file(path)
-				else: raise
-			except:
-				try:
-					path = os.path.join(sys.prefix, 'share', 'gnoduino', "pixmaps", i[1])
-					if os.path.exists(path):
-						w.set_from_file(path)
-					else: raise
-				except Exception,e:
-					print(e)
-					raise SystemExit(_("Cannot load pixmap files"))
+			w.set_from_file(misc.getPixmapPath(i[1]))
 			o = gui.get_object(i[0])
 			o.set_icon_widget(w)
 			o.show_all()
 			if i[2] != None:
 				o.connect("clicked", i[2])
+		if (sys.argv[1:]):
+			if sys.argv[1:][0] == "--help" or sys.argv[1:][0] == "-h":
+				print """--help Prints the command line options.\n--version Output version information and exit"""
+				sys.exit(0)
+			if sys.argv[1:][0] == "--version" or sys.argv[1:][0] == "-v":
+				print "gnoduino %s" % gnoduino.__version__
+				sys.exit(0)
+			for f in sys.argv[1:]:
+				processFile(f)
 		gtk.main()
 	except KeyboardInterrupt:
 		print "\nExit on user cancel."
