@@ -46,15 +46,6 @@ avr_bl = [
 ]
 
 
-def getConfigSerialPort(notify, output):
-	if config.cur_serial_port == -1:
-		misc.printError(notify, output, \
-			_("Serial port not configured! Use Tools->Serial Port to configure port."))
-		return -1
-	else:
-		return config.cur_serial_port
-
-
 def burnBootloader(serial, output, notify, id):
 	context = notify.get_context_id("main")
 	notify.pop(context)
@@ -114,14 +105,17 @@ def upload(obj, serial, output, notify):
 	notify.pop(context)
 	notify.push(context, _("Flashing..."))
 	b = board.Board()
+	port = serial.getConfigSerialPort(notify, output)
+	if port == -1:
+		notify.pop(context)
+		notify.push(context, _("Flashing error."))
+		return
 	serial.resetBoard()
 	compline=[i for i in avr]
 	# avrdude wants "stk500v1" to distinguish it from stk500v2
 	protocol = b.getPGM(b.getBoard())
 	if protocol == "stk500": protocol = "stk500v1"
 	compline.append("-c" + protocol)
-	port = getConfigSerialPort(notify, output)
-	if port == -1: raise
 	compline.append("-P" + port)
 	compline.append("-b" + b.getPGMSpeed(b.getBoard()))
 	compline.append("-p" + b.getBoardMCU(b.getBoard()))
@@ -131,7 +125,6 @@ def upload(obj, serial, output, notify):
 		run = misc.runProgOutput(output, compline)
 		if run == False: raise
 	except:
-		print "Error: %s" % e
 		notify.pop(context)
 		notify.push(context, _("Flashing error."))
 		return
